/* 
 * File:   testmain.cpp
 * Author: carlos
 *
 * Created on 7 de julio de 2010, 15:36
 */

#include <gsl/gsl_rng.h>
#include "funsoft.h"
#include "fitnessfunction.h"
#include "de_d40_mm.h"

#include <stdlib.h>
#include <iostream>
#include <cmath>
#include <limits>
#include <values.h>
#include <time.h>


using namespace std;

int semillas[] = {12345678, 23456781, 34567812, 45678123, 56781234, 67812345, 78123456, 81234567,
    12435678, 24356781, 43567812, 35678124, 56781243, 67812435, 78124356, 81243567,
    18435672, 84356721, 43567218, 35672184, 56721843, 67218435, 72184356, 21843567,
    18437652, 84376521, 43765218, 37652184, 76521843, 65218437, 52184376, 21843765,
    18473652, 84736521, 47365218, 73652184, 36521847, 65218473, 52184736, 21847365,
    15473682, 54736821, 47368215, 73682154, 36821547, 68215473, 82154736, 21547368,
    15472683, 54726831, 47268315, 72683154, 26831547, 68315472, 83154726, 31547268,
    65472183, 54721836, 47218365, 72183654, 21836547, 18365472, 83654721, 36547218,
    35472186, 54721863, 47218635, 72186354, 21863547, 18635472, 86354721, 63547218,
    35427186, 54271863, 42718635, 27186354, 71863542, 18635427, 86354271, 63542718,
    36427185, 64271853, 42718536, 27185364, 71853642, 18536427, 85364271, 53642718,
    36428175, 64281753, 42817536, 28175364, 81753642, 17536428, 75364281, 53642817,
    36528174, 65281743, 52817436, 28174365};

double pi;
double e;

int WrapperFunction::dim;
tFitness(*WrapperFunction::ff)(int, const double*);

/*
 *
 */
int main(int argc, char** argv) {

    if (argc != 4) {
        cout << argv[0] << " <funcion> <dimension> <run>" << endl;
        cout << endl;
        cout << "<funcion>: {1-19}" << endl;
        return 1;
    }

    pi = acos(-1.0);
    e = exp(1.0);

    int ifunction = atoi(argv[1]);
    int dimension = atoi(argv[2]);
    int run = atoi(argv[3]);
    int factorEvals = 5000;
    double minBound = -100;
    double maxBound = 100;
    double optimum = 0.;
    WrapperFunction *of;

    switch (ifunction) {
        case 1: of = new WrapperFunction(Shifted_Sphere, dimension, (char *) "f1"); // "Sphere");
            optimum = -450.;
            break;
        case 2: of = new WrapperFunction(Schwefel_Problem, dimension, (char *) "f2"); // "Schwefel");
            optimum = -450.;
            break;
        case 3: of = new WrapperFunction(Shifted_Rosenbrock, dimension, (char *) "f3"); // "Rosenbrock");
            optimum = 390.;
            break;
        case 4: of = new WrapperFunction(Shifted_Rastrigin, dimension, (char *) "f4"); // "Rastrigin");
            minBound = -5;
            maxBound = 5;
            optimum = -330.;
            break;
        case 5: of = new WrapperFunction(Shifted_Griewank, dimension, (char *) "f5"); // "Griewank");
            minBound = -600;
            maxBound = 600;
            optimum = -180.;
            break;
        case 6: of = new WrapperFunction(Shifted_Ackley, dimension, (char *) "f6"); // "Ackley");
            minBound = -32;
            maxBound = 32;
            optimum = -140.;
            break;
        case 7: of = new WrapperFunction(f_Schwefel2_22, dimension, (char *) "f7"); // "Schwefel2.22");
            minBound = -10;
            maxBound = 10;
            break;
        case 8: of = new WrapperFunction(f_Schwefel1_2, dimension, (char *) "f8"); // "Schwefel1.2");
            minBound = -65536;
            maxBound = 65536;
            break;
        case 9: of = new WrapperFunction(Extended_f_10, dimension, (char *) "f9"); // "EF10");
            break;
        case 10: of = new WrapperFunction(f_Bohachevsky, dimension, (char *) "f10"); // "Bohachevsky");
            minBound = -15;
            maxBound = 15;
            break;
        case 11: of = new WrapperFunction(f_Schaffer, dimension, (char *) "f11"); // "Schaffer");
            break;
        case 12: of = new WrapperFunction(f_Hybrid_12, dimension, (char *) "f12"); // "Hybrid_12");
            break;
        case 13: of = new WrapperFunction(f_Hybrid_13, dimension, (char *) "f13"); // "Hybrid_13");
            break;
        case 14: of = new WrapperFunction(f_Hybrid_14, dimension, (char *) "f14"); // "Hybrid_14");
            minBound = -5;
            maxBound = 5;
            break;
        case 15: of = new WrapperFunction(f_Hybrid_15, dimension, (char *) "f15"); // "Hybrid_15");
            minBound = -10;
            maxBound = 10;
            break;
        case 16: of = new WrapperFunction(f_Hybrid_16new, dimension, (char *) "f16"); // "Hybrid_16");
            break;
        case 17: of = new WrapperFunction(f_Hybrid_17new, dimension, (char *) "f17"); // "Hybrid_17");
            break;
        case 18: of = new WrapperFunction(f_Hybrid_18new, dimension, (char *) "f18"); // "Hybrid_18");
            minBound = -5;
            maxBound = 5;
            break;
        case 19: of = new WrapperFunction(f_Hybrid_19new, dimension, (char *) "f19"); // "Hybrid_19");
            minBound = -10;
            maxBound = 10;
            break;

        default:
            cerr << "Unknown function: " << ifunction << endl;
            return 1;
    }

    cout.setf(ios::scientific);
    clock_t start, end;
    double cpu_time_used;

    start = clock();

    DE_D40_MM mialg(of, dimension, minBound, maxBound, semillas[run]);
    long double fvalue = mialg.run(dimension * factorEvals);

    end = clock();
    cpu_time_used = ((double) (end - start)) / CLOCKS_PER_SEC;
    cout.setf(ios::scientific);
    cout << (fvalue - optimum) << "\t" << cpu_time_used << endl;

    delete of;
    return (EXIT_SUCCESS);
}
